<?php

namespace App\Actions;

use App\Http\Resources\WithdrawRequestResource;
use App\Models\Token;
use App\Models\User;
use App\Models\WithdrawRequest;
use Genocide\Radiocrud\Exceptions\CustomException;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class WithdrawRequestAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(WithdrawRequest::class)
            ->setResource(WithdrawRequestResource::class)
            ->setValidationRules([
                'get_query' => [
                    'first_name' => 'string|max:100',
                    'last_name' => 'string|max:100',
                    'status' => 'string',
                    'amount' => 'int',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'change-status' => [
                    'status' => 'required|string',
                    'tracing_code' => 'string'
                ],
                'new-request' => [
                    'amount' => 'required|int'
                ]
            ])
            ->setQueryToEloquentClosures([
                'first_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.name', $query['first_name']);
                    });
                },
                'last_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.lastname', $query['last_name']);
                    });
                },
                'amount' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('amount', $query['amount']);
                    });
                },
                'status' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('status', $query['status']);
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }

    public function changeStatusById($id)
    {
        $status = $this->getDataFromRequest()['status'];
        $tracingCode = null;
        if(isset($this->getDataFromRequest()['tracing_code']))
        {
            $tracingCode = $this->getDataFromRequest()['tracing_code'];
        }
        $withdrawRequest = WithdrawRequest::query()->where('id', $id)->first();
        $user = User::query()->where('id', $withdrawRequest->user_id)->first();

        if($status == 'accept')
        {
            if($user->withdrawable_amount >= $withdrawRequest->amount)
            {
                $user->wallet_amount -= $withdrawRequest->amount;
                $user->withdrawable_amount -= $withdrawRequest->amount;
                $withdrawRequest->status = 'accepted';
                $withdrawRequest->tracing_code = $tracingCode;
                $user->save();
                $withdrawRequest->save();
                return "عمليات كسر از كيف پول با موفقيت انجام شد ";
            }
            return "موجودي كيف پول كاربر كافي نميباشد";
        }

        if($status == 'reject')
        {
            $withdrawRequest->status = 'rejected';
            $withdrawRequest->save();
            return "درخواست برداشت كاربر از كيف پول لغو شد";
        }
        return "something went wrong";
    }

    public function newRequest($token)
    {
        $user_id = Token::query()->where('token', $token)->first()->user_id;
        $user = User::query()->where('id', $user_id)->first();
        $amount = $this->getDataFromRequest()['amount'];

        $recentRequests = WithdrawRequest::query()->where('user_id', $user_id)->where('status', 'pending')->first();
        if($user->withdrawable_amount < $amount)
        {
            throw new CustomException('موجودي كيف پول شما كافي نمي باشد',400,400);

        }

        if(!empty($recentRequests))
        {
            throw new CustomException('درخواست قبلي شما در انتظار تاييد مي باشد',400,400);
        }
        else
        {
            WithdrawRequest::query()->create([
                'user_id' => $user_id,
                'amount' => $amount,
                'status' => 'pending'
            ]);
            return "درخواست شما ثبت شد";
        }
    }

    public function withdrawHistory($token)
    {
        $user_id = Token::query()->where('token', $token)->first()->user_id;
        $user = User::query()->where('id', $user_id)->first();

        return WithdrawRequest::query()->where('user_id', $user_id)->where('status', 'accepted')->get();
    }
}
